var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { EXPLORER_GQL_API, DEFAULT_QUERY_RESPONSE, } from "../gqlutils.js";
import fetch from "node-fetch";
/**
 * @notice Builds string with txn parameters for a GraphQL query
 * @param chainId - Chain ID. Optional.
 * @param address - Address. Optional.
 * @param txnHash - Transaction hash. Optional.
 * @param kappa - Kappa. Optional.
 * @returns String with txn parameters for a GraphQL query
 */
function buildTxnParams(chainId, address, txnHash, kappa) {
    if (!chainId && !address && !txnHash && !kappa) {
        throw new Error("Must provide at least one of chainId, address, txnHash, kappa");
    }
    var query_params = "";
    if (chainId) {
        query_params += "chainId: ".concat(chainId);
    }
    if (address) {
        if (query_params.length > 0) {
            query_params += ", ";
        }
        query_params += "address: \"".concat(address, "\"");
    }
    if (txnHash) {
        if (query_params.length > 0) {
            query_params += ", ";
        }
        query_params += "txnHash: \"".concat(txnHash, "\"");
    }
    if (kappa) {
        if (query_params.length > 0) {
            query_params += ", ";
        }
        query_params += "kappa: \"".concat(kappa, "\"");
    }
    return query_params;
}
/**
 * @notice Builds a GraphQL query string for the given query type
 * @param chainId The chainId to query for. Optional.
 * @param address The address to query for. Optional.
 * @param txnHash The transaction hash to query for. Optional.
 * @param kappa The kappa to query for. Optional.
 * @returns A GraphQL query string
 */
function buildTxnQuery(chainId, address, txnHash, kappa) {
    // Get the parameters of the query
    var query_params = buildTxnParams(chainId, address, txnHash, kappa);
    // Construct the query
    var query = "\n      query{\n          bridgeTransactions(".concat(query_params, ") {\n              ").concat(DEFAULT_QUERY_RESPONSE, "\n          }\n      }\n  ");
    return query;
}
// - [EXTERNAL] - //
/**
 * @notice Gets transaction info with a GraphQL query
 * @param chainId The chainId to query for. Optional.
 * @param address The address to query for. Optional.
 * @param txnHash The transaction hash to query for. Optional.
 * @param kappa The kappa to query for. Optional.
 * @returns A promise that resolves to the transaction info as a JSON object
 */
export function getBridgeTxnInfo(args) {
    return __awaiter(this, void 0, void 0, function () {
        var query, response, error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    query = buildTxnQuery(args.chainId, args.address, args.txnHash, args.kappa);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 4, , 5]);
                    return [4 /*yield*/, fetch(EXPLORER_GQL_API, {
                            method: "post",
                            body: JSON.stringify({ query: query }),
                            headers: { "Content-Type": "application/json" },
                        })];
                case 2:
                    response = _a.sent();
                    return [4 /*yield*/, response.json()];
                case 3: return [2 /*return*/, (_a.sent())];
                case 4:
                    error_1 = _a.sent();
                    throw new Error("Error fetching bridge transactions: ".concat(error_1));
                case 5: return [2 /*return*/];
            }
        });
    });
}
